package com.doverfuelingsolutions.issp.view.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.setFragmentResult
import androidx.recyclerview.widget.GridLayoutManager
import com.doverfuelingsolutions.issp.R
import com.doverfuelingsolutions.issp.databinding.FragmentNozzleBinding
import com.doverfuelingsolutions.issp.fusion.FusionManager
import com.doverfuelingsolutions.issp.view.adapter.ChooseListAdapter
import com.scwang.smart.refresh.header.ClassicsHeader
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch

class FragmentNozzle : FragmentBasic() {

    private lateinit var binding: FragmentNozzleBinding

    private val pumpList: List<String> by lazy { FusionManager.pumpList.map { it.nozzleList.joinToString() } }
    private val chooseListAdapter: ChooseListAdapter<String> by lazy {
        ChooseListAdapter(
            pumpList,
            this::select
        )
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        if (!this::binding.isInitialized) {
            binding = DataBindingUtil.inflate(inflater, R.layout.fragment_nozzle, container, false)

            binding.nozzleList.layoutManager = GridLayoutManager(context, 4)
            binding.nozzleList.adapter = chooseListAdapter

            binding.smartRefreshLayout.setEnableLoadMore(false)
            binding.smartRefreshLayout.setRefreshHeader(ClassicsHeader(context))
            binding.smartRefreshLayout.setOnRefreshListener { handleNozzleRefresh() }

            setToolBar(binding.fragmentToolbar)
        }
        return binding.root
    }

    private fun handleNozzleRefresh() {
        GlobalScope.launch(Dispatchers.Main) {
            val resultNozzles = FusionManager.requestPumpInfo()
            if (resultNozzles.success && resultNozzles.data != null) {
                if (FusionManager.pumpList.isNotEmpty()) FusionManager.pumpList.clear()
                FusionManager.pumpList.addAll(resultNozzles.data)
                binding.nozzleList.adapter?.notifyDataSetChanged()
                binding.smartRefreshLayout.finishRefresh(200, true, false)
            } else {
                binding.smartRefreshLayout.finishRefresh(200, false, true)
            }
        }
    }

    private fun select(nozzle: String) {
        setFragmentResult(
            FragmentConstants.fragmentEvent,
            bundleOf(
                Pair(FragmentConstants.eventType, FragmentConstants.eventOrderList),
                Pair(FragmentConstants.searchType, FragmentConstants.typeNozzle),
                Pair(FragmentConstants.pumpId, FusionManager.pumpList.find {
                    it.nozzleList.joinToString() == nozzle
                }!!.pumpId),
            )
        )
    }
}