package com.doverfuelingsolutions.issp.view.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.doverfuelingsolutions.issp.R
import com.doverfuelingsolutions.issp.data.GlobalData
import com.doverfuelingsolutions.issp.databinding.FragmentNozzleOrdersBinding
import com.doverfuelingsolutions.issp.fusion.FusionManager
import com.doverfuelingsolutions.issp.utils.log.DFSLog
import com.doverfuelingsolutions.issp.view.adapter.OrderListAdapter
import com.google.android.material.snackbar.Snackbar
import com.scwang.smart.refresh.header.ClassicsHeader
import com.wayne.www.waynelib.fdc.message.DeviceClass
import kotlinx.coroutines.launch

class FragmentOrderList : FragmentBasic() {

    private lateinit var binding: FragmentNozzleOrdersBinding
    private val orderList = arrayListOf<DeviceClass>()
    private val orderListAdapter: OrderListAdapter by lazyOf(OrderListAdapter(orderList, this::select))

    private val snackbar: Snackbar by lazy {
        Snackbar.make(binding.smartRefreshLayout, "", 4000)
            .apply { anchorView = binding.smartRefreshLayout }
    }
    val viewModel = FragmentNozzleViewModel()

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        if (!this::binding.isInitialized) {
            binding = DataBindingUtil.inflate(inflater, R.layout.fragment_nozzle_orders, container, false)

            binding.orderList.layoutManager = GridLayoutManager(context, 2)
            binding.orderList.adapter = orderListAdapter

            binding.smartRefreshLayout.setEnableLoadMore(false)
            binding.smartRefreshLayout.setRefreshHeader(ClassicsHeader(context))
            binding.smartRefreshLayout.setOnRefreshListener { handleOrdersRefresh() }

            binding.lifecycleOwner = this
            binding.viewModel = viewModel

            setToolBar(binding.fragmentToolbar)
        }
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // TODO 订单状态：是否需要对一些已锁定、已清理的订单做处理？
        lifecycleScope.launch {
            binding.orderList.scrollTo(0, 0)
            // local data
            val localList = filterByMode(GlobalData.orderList.get())
            if (orderList.isNotEmpty()) orderList.clear()
            orderList.addAll(localList)
            orderListAdapter.notifyDataSetChanged()

            // remote data
            val result = FusionManager.getAllUnsettledOrder(viewModel.pump.value!!)
            if (result.success && result.data != null) {
                val localHash = localList.joinToString { it.transactionSeqNo }
                val remoteHash = result.data.joinToString { it.transactionSeqNo }
                if (localHash != remoteHash) {
                    if (orderList.isNotEmpty()) orderList.clear()
                    orderList.addAll(filterByMode(result.data))
                    orderListAdapter.notifyDataSetChanged()

                    snackbar.setText(R.string.data_refresh).show()
                }
            }
        }
    }

    private fun handleOrdersRefresh() {
    }

    private fun select() {
        DFSLog.i("select")
    }

    private fun filterByMode(orders: List<DeviceClass>): List<DeviceClass> {
        val isNozzle = viewModel.type.value == FragmentConstants.typeNozzle
        val pump = viewModel.pump.value!!
        return orders.filter {
            if (isNozzle) {
                it.pumpNo == pump
            } else {
                // TODO 油品选择
                true
            }
        }
    }

    class FragmentNozzleViewModel : ViewModel() {
        val type = MutableLiveData(FragmentConstants.typeNozzle)
        val pump = MutableLiveData(-1)
        val oilName = MutableLiveData("")
    }
}